program jcCalibrate
  ! Program for calibrating P and Z components
  ! Primarily for PZ summation
  ! Assumes that instruement responses have already been removed
  ! Wants to account for differences in component coupling
  ! Based on Wiener filtering procedure (Yilmaz, Seismic Data Analysis)
  ! Imposes Hann filter for time window
  !
  ! Inputs:
  !   1) pfile: hydrophone data
  !   2) zfile: vertical geophone data
  !   3) in_hff: header information
  !        note -- assumes header is same for P and Z
  !   4) picks: pick text file
  !   5) lcorr: length of correlations (in seconds?)
  !   6) lfilter: length of filter (in seconds?)
  !   7) sxkey [32]
  !   8) sykey [33]
  !   9) tkey [1]
  !  10) pxkey[2] 
  !  11) pykey[2] 
  !  12) eps: noise to add
  !
  ! Outputs:
  !   1) calibration filter (to be applied to Z)
  !   2) out_hff: output header file
  !
  ! Jason Chang
  ! January 2016

  !======================================
  ! standard declarations
  !======================================
  use sep
  use read_picks_mod
  use correlation_mod
  use taper_mod
  use shaping_mod
  implicit none


  !======================================
  ! declare variables
  !======================================
  ! program variables
  integer                         :: ii,idiff,ix0
  integer                         :: it1,it2,temp
  real                            :: diff,mdiff
  real                            :: vals
  real                            :: eps

  ! hydrophone variables
  integer                         :: np1,np2
  real                            :: op1,dp1
  real,allocatable,dimension(:)   :: pp

  ! vertical variables
  integer                         :: nz1,nz2
  real                            :: oz1,dz1
  real,allocatable,dimension(:)   :: zz

  ! header variables
  integer                         :: nh1,nh2
  integer                         :: sxkey,sykey
  real,allocatable,dimension(:,:) :: header

  ! filter variables
  integer                         :: ncorr,nfilter
  real                            :: lcorr,lfilter
  real,allocatable,dimension(:)   :: zzauto,pzcross,filter

  ! picks variables
  integer                         :: tkey,pxkey,pykey
  real                            :: t1,t2,sx1,sx2,sy1,sy2
  real,allocatable,dimension(:,:) :: picks
  character(len=30)               :: pickfile


  !======================================
  ! open I/O
  !======================================
  call sep_init()

  ! hydrophone parameters
  if (exist_file('pfile')) then
    call from_aux('pfile','n1',np1,1 )
    call from_aux('pfile','n2',np2,1 )
    call from_aux('pfile','o1',op1,0.)
    call from_aux('pfile','d1',dp1,1.)
  else
    call seperr('Need pressure history file pfile=')
  end if

  ! vertical parameters
  if (exist_file('zfile')) then
    call from_aux('zfile','n1',nz1,1 )
    call from_aux('zfile','n2',nz2,1 )
    call from_aux('zfile','o1',oz1,0.)
    call from_aux('zfile','d1',dz1,1.)
  else
    call seperr('Need vertical history file zfile=')
  end if
  if (np2.ne.nz2) call seperr('Size of P and Z do not match')

  ! header parameters
  if (exist_file('in_hff')) then
    call from_aux('in_hff','n1',nh1,1 )
    call from_aux('in_hff','n2',nh2,1 )
  else
    call seperr('Need header history file in_hff=')
  end if
  call from_param('sxkey',sxkey,32)
  call from_param('sykey',sykey,33)
  if (nh2.ne.nz2) call seperr('Size of header and data do not match')

  ! filter parameters
  call from_param('lcorr',lcorr,dp1*(np1-1))
  call from_param('ncorr',ncorr,int(1.5+(lcorr/dp1)))
  call from_param('lfilter',lfilter,dp1*(np1-1))
  call from_param('nfilter',nfilter,int(1.5+(lfilter/dp1)))
  call from_param('eps',eps,0.1)

  ! pick parameters
  if (exist_file('picks')) then
    call from_param('picks',pickfile)
  else
    call seperr('Need picks text file picks=')
  end if
  call from_param('tkey' ,tkey ,1)
  call from_param('pxkey',pxkey,2)
  call from_param('pykey',pykey,3)

  ! output history parameters
  call to_history('n1',nfilter)
  call to_history('n2',1  )
  call to_history('o1',op1)
  call to_history('o2',0. )
  call to_history('d1',dp1)
  call to_history('d2',1. )

  ! output header parameters
  if (exist_file('out_hff')) then
    call to_history('n1',nh1,'out_hff')
    call to_history('o1',0. ,'out_hff')
    call to_history('d1',1. ,'out_hff')
    call to_history('n2',1  ,'out_hff')
    call to_history('o2',0. ,'out_hff')
    call to_history('d2',1. ,'out_hff')
  end if


  !======================================
  ! initializations
  !======================================
  ! subroutine to read text file
  call read_picks(pickfile,picks)
  if (size(picks,1).ne.2) call seperr('Should only be two picks in file')
  ! grab relevant values from pick file
  ! times
  t1 = picks(1,tkey)
  t2 = picks(2,tkey)
  it1 = 1.5+(t1-op1)/dp1
  it2 = 1.5+(t2-op1)/dp1
  if (it1.gt.it2) then
    temp = it1
    it1  = it2
    it2  = temp
  end if
  ! source locations
  sx1 = picks(1,pxkey)
  sx2 = picks(2,pxkey)
  sy1 = picks(1,pykey)
  sy2 = picks(2,pykey)
  if (sx1.ne.sx2) call seperr('Picks should have same sx')
  if (sy1.ne.sy2) call seperr('Picks should have same sy')

  ! read in data and header
  allocate(pp(np1))
  allocate(zz(nz1))
  allocate(header(nh1,nh2))
  call sreed('in_hff',header,4*nh1*nh2)

  ! initialize taper and correlation
  call taper_init(np1,op1,dp1,3)
  call correlation_init(np1)
  call shaping_init(nfilter)

  ! allocate vectors
  allocate(pzcross(np1))
  allocate(zzauto(np1))
  allocate(filter(nfilter))


  !======================================
  ! kernel
  !======================================
  ! use header to determine which trace is closest to source
  mdiff = huge(0.0)
  do ii=1,nh2
    diff = sqrt((header(sxkey,ii)-sx1)**2 + (header(sykey,ii)-sy1)**2)
    if (diff.lt.mdiff) then
      idiff = ii-1
      mdiff = diff
    end if
  end do

  ! seek to the point in data
  ! pressure
  call sseek('pfile',4*np1*idiff,0)
  call sreed('pfile',pp,4*np1)
  ! vertical
  call sseek('zfile',4*nz1*idiff,0)
  call sreed('zfile',zz,4*nz1)

  ! taper around reflection window
  call apply_taper(pp,t1,t2,0.,0.)
  call apply_taper(zz,t1,t2,0.,0.)

  ! calculate crosscorrelation of P and Z
  ! calculate autocorrelation of Z
  call correlation(zz,zz,zzauto)
  call correlation(pp,zz,pzcross)

  ! calculate zero lag index
  ix0 = (np1/2)+1

  ! calculate wiener filter
  call wiener_filter(pzcross(ix0:1:-1),zzauto(ix0:1:-1),filter(:),eps)

  ! write out file
  call srite('out',filter,4*nfilter)
  if (exist_file('out_hff')) then
    call srite('out_hff',header(:,idiff+1),4*nh1)
  end if  


  !======================================
  ! clean up
  !======================================
  call sep_close()
  call correlation_close()
  call taper_close()
  call shaping_close()
  deallocate(picks,header,pp,zz,pzcross,zzauto,filter)
  call exit(0)

end program
