program jcPWStack
  ! Program to perform phase-weighted stack (Schimmel and Paulssen 1997)
  !
  ! Notes:
  !   Assumes that the input is a regular file (i.e., after Cat3d)
  !   Only stacks on the second axis
  !
  ! Input:
  !   default: data cube
  !   normalize [1]: whether to normalize by n2 or not
  !   power [0]: power to raise weight to (zero is linear stack)
  !
  ! Output:
  !   default: phase weighted stack
  !
  ! Jason Chang
  ! January 2017


  !---------------------------------------------
  ! imports and calls
  !---------------------------------------------
  use sep
  use bandpass_mod
  implicit none


  !---------------------------------------------
  ! declare variables
  !---------------------------------------------
  ! program parameters
  integer                            :: ii,jj,kk
  integer                            :: rc,wc,srite,sreed
  integer                            :: topt
  integer                            :: ec

  ! option parameters
  real                               :: power
  logical                            :: normalize

  ! input parameters
  integer                            :: n1,n2,n3,n4
  real                               :: d1,o1
  real,allocatable,dimension(:)      :: datain
  complex,allocatable,dimension(:)   :: cdatain

  ! summation parameters
  real                               :: weight
  real,allocatable,dimension(:)      :: datasum
  real,allocatable,dimension(:)      :: phase
  complex,allocatable,dimension(:)   :: cphasesum
  real,allocatable,dimension(:)      :: weights
  


  !---------------------------------------------
  ! allocate variables
  !---------------------------------------------
  call sep_init()

  ! input parameters
  call from_history('n1',n1,1)
  call from_history('d1',d1,0.002)
  call from_history('o1',o1,0.)
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('n4',n4,1 )

  ! optional paramters
  call from_param('normalize',normalize,.true.)
  call from_param('topt',topt,0)
  call from_param('power',power,0.)

  ! output parameters
  call to_history('n1',n1)
  call to_history('d1',d1)
  call to_history('o1',o1)
  call to_history('n2',n3)
  call to_history('n3',n4)
  call to_history('n4',1)

  if (exist_file('env_out')) then
      call to_history('n1',n1,'env_out')
      call to_history('d1',d1,'env_out')
      call to_history('o1',o1,'env_out')
      call to_history('n2',n3,'env_out')
      call to_history('n3',n4,'env_out')
      call to_history('n4',1,'env_out')
  end if

  if (exist_file('weight_out')) then
      call to_history('n1',n1,'weight_out')
      call to_history('d1',d1,'weight_out')
      call to_history('o1',o1,'weight_out')
      call to_history('n2',n3,'weight_out')
      call to_history('n3',n4,'weight_out')
      call to_history('n4',1,'weight_out')
  end if


  !---------------------------------------------
  ! initialize arrays and subroutines
  !---------------------------------------------
  allocate(datain(n1))
  allocate(cdatain(n1))
  allocate(datasum(n1));
  allocate(phase(n1))
  allocate(cphasesum(n1));
  allocate(weights(n1));
  call bandpass_init(n1,o1,d1,topt)


  ! loop over third axis and above
  do ii=1,n3*n4
      ! reset variables
      datasum = 0.
      cphasesum = 0.

      ! loop over stacking axis
      do jj=1,n2
          ! read in trace
          rc = sreed('in',datain(:),n1*4)
    
          ! sum trace
          datasum(:) = datasum(:)+datain(:)

          ! create analytic signal
          cdatain = cmplx(datain(:),0.)
          call apply_bandpass(cdatain,0.,0.)

          ! calculate phase of the analytic signal
          do kk=1,n1
              phase(kk) = atan2(aimag(cdatain(kk)),real(cdatain(kk)))
              cphasesum(kk) = cphasesum(kk) + exp(cmplx(0.,phase(kk)))
              !if (kk==1) then
              !    write(0,*) cdatain(kk)
              !    write(0,*) aimag(cdatain(kk))
              !    write(0,*) real(cdatain(kk))
              !    write(0,*) phase(kk)
              !    write(0,*) cphasesum(kk)
              !end if
          end do

      end do

      ! multiply the stack by the phase stack component (element by element)
      do kk=1,n1
          !weight = (cabs(cphasesum(kk))/n2)**power
          !datasum(kk) = weight*datasum(kk)
          weights(kk) = (cabs(cphasesum(kk))/n2)**power
          datasum(kk) = weights(kk)*datasum(kk)
      end do

      ! normalize or not
      if (normalize) then
          wc = srite('out',datasum(:)/n2,n1*4)
      else
          wc = srite('out',datasum(:),n1*4)
      end if

      !if (exist_file('env_out')) then
      !    cdatain = cmplx(datasum(:),0.)
      !    call apply_bandpass(cdatain,0.,0.)
      !    ec = srite('env_out',cabs(cdatain(:)),4*n1)
      !end if

      if (exist_file('weight_out')) then
          ec = srite('weight_out',weights,4*n1)
      end if


  end do

  !---------------------------------------------
  ! clean up
  !---------------------------------------------
  deallocate(datain,cdatain,datasum,phase,cphasesum,weights)
  call bandpass_close()
  call sep_close()
  call exit(0)

end program
