program jcPZsum
  ! Program to perform PZ summation
  ! Inputs are ideally receiver gathers
  ! Assumes that filter inputs contain all receivers with headers
  !
  ! Inputs:
  !   1) pfile: history file for pressure component
  !   2) zfile: history file for vertical component
  !   3) hff: header file for receivers
  !   4) ffile: history file for calibration filters for each receiver
  !        note: this is derived from active source data (via Ettore)
  !   5) fhff: header file for filter
  !   6) water_vel: water velocity
  !   7) eps: scaling factor for calibration filter
  !
  ! Outputs:
  !   1) upfile: history file for upgoing wavefield
  !   2) downfile: history file for downgoing wavefield
  !   3) zffile: vertical after calibration filter applied
  !
  ! Jason Chang
  ! October 2014
  ! Update: January 2016
  !   1) fixed some of the comments
  !   2) changed module from fourier to fourier1d
  !   3) changed fheader from integer to real array
  !   4) changed the velocity parameter in PZ sum
  !        - currently set to just water velocity

  !==================================
  ! standard declarations
  !==================================
  use sep
  use fourier1d_mod
  implicit none

  !==================================
  ! declare variables
  !==================================
  ! program variables
  integer                            :: ii,jj
  integer                            :: ifilter
  integer                            :: ntraces
  real                               :: water_vel,vert_slow,vfactor
  real                               :: eps

  ! pressure file parameters
  integer                            :: n1p,n2p
  real                               :: o1p,d1p
  real,allocatable,dimension(:)      :: pdata

  ! vertical file parameters
  integer                            :: n1z,n2z
  real                               :: o1z,d1z
  real,allocatable,dimension(:)      :: zdata
  complex,allocatable,dimension(:)   :: czdata

  ! reciever header file parameters
  integer                            :: n1hdr,n2hdr
  integer                            :: gxkey,gykey
  real                               :: gx,gy
  real,allocatable,dimension(:)      :: header

  ! filter file parameters
  integer                            :: n1f,n2f
  real                               :: o1f,d1f
  real,allocatable,dimension(:,:)    :: filter
  complex,allocatable,dimension(:)   :: cfilter

  ! filter file header parameters
  integer                            :: n1fhdr,n2fhdr
  integer                            :: gxfkey,gyfkey,velfkey
  real,allocatable,dimension(:,:)    :: fheader

  ! output file parameters
  integer                            :: n1,n2
  real                               :: o1,o2,d1,d2  
  real,allocatable,dimension(:)      :: upgoing,downgoing
  

  !==================================
  ! open I/O
  !==================================
  call sep_init()

  ! input parameters
  call from_param('water_vel',water_vel,1475.)

  ! pressure file parameters
  if (exist_file('pfile')) then
    call from_aux('pfile','n1',n1p,1 )
    call from_aux('pfile','o1',o1p,0.)
    call from_aux('pfile','d1',d1p,1.)
    call from_aux('pfile','n2',n2p,1 )
  else
    call seperr('Need pressure component file')
  end if

  ! vertical file parameters
  if (exist_file('zfile')) then
    call from_aux('zfile','n1',n1z,1 )
    call from_aux('zfile','o1',o1z,0.)
    call from_aux('zfile','d1',d1z,1.)
    call from_aux('zfile','n2',n2z,1 )
  else
    call seperr('Need pressure component file')
  end if

  ! receiver header information
  if (exist_file('hff')) then
    call from_aux('hff','n1',n1hdr)
    call from_aux('hff','n2',n2hdr)
  else
    call seperr('Need header info for receivers')
  end if
  call from_param('gxkey',gxkey,1)
  call from_param('gykey',gykey,2)

  ! filter file parameters
  if (exist_file('ffile')) then
    call from_aux('ffile','n1',n1f)
    call from_aux('ffile','o1',o1f)
    call from_aux('ffile','d1',d1f)
    call from_aux('ffile','n2',n2f)
  else
    call seperr('Need calibration filter file')
  end if
  call from_param('eps',eps,1.)

  ! filter header file parameters
  if (exist_file('fhff')) then
    call from_aux('fhff','n1',n1fhdr)
    call from_aux('fhff','n2',n2fhdr)
  else
    call seperr('Need header info for filter coefficients')
  end if
  call from_param('gxfkey' ,gxfkey,1)
  call from_param('gyfkey' ,gyfkey,2)
  call from_param('velfkey',velfkey,2)


  ! check that parameters match
  if (n1z.ne.n1p) call seperr('n1 does not match')
  n1 = n1z
  if (o1z.ne.o1p) call seperr('o1 does not match')
  o1 = o1z
  if (d1z.ne.d1p) call seperr('d1 does not match')
  d1 = d1z
  if (n2z.ne.n2p) call seperr('n2 does not match')
  n2 = n2z
  if (n2.ne.n2hdr) call seperr('n2 data does not match header')
  if (n2f.ne.n2fhdr) call seperr('n2 filter does not match header')
  
  ! output upgoing file
  if (.not.exist_file('upfile')) call seperr('Need output upgoing file')
  
  ! output downgoing file
  if (.not.exist_file('downfile')) call seperr('Need output downgoing file')


  !==================================
  ! allocate
  !==================================
  allocate(pdata(n1))
  allocate(zdata(n1))
  allocate(czdata(n1))
  allocate(upgoing(n1))
  allocate(downgoing(n1))
  allocate(header(n1hdr))
  allocate(filter(n1f,n2f))
  allocate(cfilter(n1))
  allocate(fheader(n1fhdr,n2fhdr))


  !==================================
  ! initialize
  !==================================
  call fft1d_init(n1)

  !==================================
  ! kernel
  !==================================
  ! read in all filter information
  call sreed('ffile',filter,4*n1f*n2f)
  call sreed('fhff',fheader,4*n1fhdr*n2fhdr)

  ! loop over each source
  ntraces = 0
  do ii = 1,n2
    ! read input files and header
    call sreed('pfile',pdata,4*n1) !pressure component
    call sreed('zfile',zdata,4*n1) !vertical component
    call sreed('hff',header,4*n1hdr) !header file

    ! find the coordinates of the current receiver
    gx = header(gxkey)
    gy = header(gykey)

    ! find corresponding filter index
    ifilter = 0
    do jj = 1,n2fhdr
      if (fheader(gxfkey,jj).eq.gx .and. fheader(gyfkey,jj).eq.gy) then
        ifilter = jj
        exit
      end if
    end do
    ! if nothing is found, then skip
    if (ifilter.eq.0) cycle
    ntraces = ntraces+1

    ! turn filter complex
    cfilter = 0.
    cfilter(1:n1f) = cmplx(filter(:,ifilter),0.)
    call fft1d(.false.,1.,cfilter)
    ! turn vertical component complex
    czdata = 0.
    czdata = cmplx(zdata,0.)
    call fft1d(.false.,1.,czdata)

    ! apply filter in the frequency domain
    czdata = eps*cfilter*czdata*sqrt(float(n1))
    call fft1d(.true.,1.,czdata)
    zdata = real(czdata)

    !! adjust the slowness parameter (was taken into account in the filter)
    !vert_slow = sqrt(water_vel**-2 - real(fheader(velfkey,ifilter)**-2))
    !vfactor = water_vel*vert_slow
    !vfactor = 1./(water_vel*water_vel)
    vfactor = 1.

    ! perform pz summation
    upgoing   = 0.5*((pdata)+(vfactor*zdata))
    downgoing = 0.5*((pdata)-(vfactor*zdata))

    ! write out files
    call srite('upfile',upgoing,4*n1)
    call srite('downfile',downgoing,4*n1)
    if (exist_file('zffile')) then
      call srite('zffile',zdata,4*n1)
    end if

  end do

  
  !==================================
  ! write out
  !==================================
  ! output upgoing file
  call to_history('n1',n1,'upfile')
  call to_history('o1',o1,'upfile')
  call to_history('d1',d1,'upfile')
  call to_history('n2',ntraces,'upfile')

  ! output downgoing file
  call to_history('n1',n1,'downfile')
  call to_history('o1',o1,'downfile')
  call to_history('d1',d1,'downfile')
  call to_history('n2',ntraces,'downfile')

  ! optional filtered vertical
  if (exist_file('zffile')) then
    call to_history('n1',n1,'zffile')
    call to_history('o1',o1,'zffile')
    call to_history('d1',d1,'zffile')
    call to_history('n2',ntraces,'zffile')
  end if


  !==================================
  ! clean up
  !==================================
  call sep_close()
  call fft1d_close()
  deallocate(pdata,zdata,filter,header,fheader)
  deallocate(czdata,cfilter)
  deallocate(upgoing,downgoing)
  call exit(0)

end program
