program jcXcorr_Forties
  ! Program for cross-correlations
  !   currently written as a trace-by-trace correlation
  !   i.e., trace 1 up is only correlated with trace 1 down
  !
  ! Inputs:
  !   1) R: receiver file
  !   2) S: source file
  !   3) nlag: number of samples for lag time
  !   4) omp_flag: whether to implement OMP
  !   5) mode: for output
  !        [1]: assumes input is sorted by time
  !             input second axis is number of receivers/sources
  !             output second axis is nrec*nsrc
  !         2 : assumes input is sorted by time
  !             input second axis is number of receivers/sources
  !             output second axis is nrec, third axis is nsrc
  !         3 : assumes input is for a given receiver
  !             input second axis is number of patches for receiver
  !             output second axis is same as number of input patches
  !
  ! Output:
  !   1) cross-correlation result
  !
  ! Jason Chang
  ! October 2014

  ! standard declaration
  use sep
  use xcorrelation_mod
  implicit none

  ! declare variables
  ! program variables
  integer                             :: ii,jj,kk
  integer                             :: ith,nth

  ! input file parameters
  integer                             :: nr1,nr2,ns1,ns2
  real                                :: or1,or2,os1,os2
  real                                :: dr1,dr2,ds1,ds2
  real,allocatable,dimension (:)      :: dataR_t,dataS_t,dataOut_t

  ! fourier parameters
  integer                             :: nlag,nt

  ! open I/O
  call sep_init()

  ! get inputs from receiver side
  if (exist_file('R')) then
    call from_aux('R','n1',nr1,1 )
    call from_aux('R','n2',nr2,1 )
    call from_aux('R','d1',dr1,1.)
    call from_aux('R','d2',dr2,1.)
    call from_aux('R','o1',or1,1.)
    call from_aux('R','o2',or2,1.)
  else
    call seperr('Need file containing receivers')
  end if

  ! get inputs from source side
  if (exist_file('S')) then
    call from_aux('S','n1',ns1,1 )
    call from_aux('S','n2',ns2,1 )
    call from_aux('S','d1',ds1,1.)
    call from_aux('S','d2',ds2,1.)
    call from_aux('S','o1',os1,1.)
    call from_aux('S','o2',os2,1.)
  else
    call seperr('Need file containing virtual sources')
  end if

  ! check input lengths for FFT
  if (floor(real(nr1)/2.).ne.ceiling(real(nr1)/2.)) call seperr("FileR should be divisible by 2")
  if (floor(real(ns1)/2.).ne.ceiling(real(ns1)/2.)) call seperr("FileS should be divisible by 2")

  ! check that the inputs are the same length
  if (nr1.ne.ns1) call seperr('nr1 and ns1 are not the same length')
  if (dr1.ne.ds1) call seperr('dr1 and ds1 are not the same length')
  if (or1.ne.os1) call seperr('or1 and os1 do not start at the same time')
  ! for MoereVest, need second axes to be the same
  if (nr2.ne.ns2) call seperr('nr2 and ns2 are not the same length')

  ! calculate the output lag times
  call from_param('nlag',nlag,nr1)

  ! output parameters
  call to_history('n1',2*nlag+1  ) !is there a +1 here?
  call to_history('o1', -nlag*dr1)
  call to_history('d1',       dr1)
  call to_history('n2',nr2)
  call to_history('o2',or2)
  call to_history('d2',dr2)

  ! prepare fourier parameters
  nt = 2*ns1+1  !long enough for total correlation

  ! allocate
  allocate(dataR_t  (nt)); dataR_t=0.
  allocate(dataS_t  (nt)); dataS_t=0.
  allocate(dataOut_t(nt)); dataOut_t=0.


  ! kernel
  !=======================================================
  ! NEW APPROACH
  !=======================================================
  ! initialize cross-correlation parameters
  !call xcorr_init(nr1,or1,dr1,nr2,or2,dr2)
  call xcorr_init(nt)
  !   includes: nlag,whether you want to perform fft shift,omp option

  ! run the cross-correlation command
  ! loop over the second axis of every trace
  do ii =1,nr2
    dataR_t = 0.
    dataS_t = 0.
    ! read in two files to cross-correlated
    call sreed('R',dataR_t(1:nr1),4*nr1)
    call sreed('S',dataS_t(1:nr1),4*ns1)

    ! perform cross-correlation
    call xcorr(dataR_t,dataS_t,dataOut_t)

    ! write out
    !call srite('out',dataOut_t(nlag:2*nlag),4*(2*nlag+1))
    call srite('out',dataOut_t(nt/2+1-nlag:(nt/2)+(2*nlag+1)),4*(2*nlag+1))

  end do
  

  ! clean up
  call sep_close()
  call xcorr_close()
  deallocate(dataR_t,dataS_t,dataOut_t)
  call exit(0)

end program
