module shaping_mod
  ! Module for shaping filters
  ! Made primarily for wavelet shaping (Wiener)
  !
  ! Jason Chang
  ! January 2016

  !=========================================
  ! includes
  !=========================================
  implicit none


  !=========================================
  ! declare module variables
  !=========================================
  integer                     :: n1


  contains


  !=========================================
  ! initialize shaping
  !=========================================
  subroutine shaping_init(n1_in)
    ! declare variables
    integer                   :: n1_in

    ! pass variables
    n1 = n1_in !filter length
    
  end subroutine


  !=========================================
  ! close shaping
  !=========================================
  subroutine shaping_close()
  end subroutine


  !=========================================
  ! wiener filter
  !   uses Levinson recursion
  !   Yilmaz, Seismic Data Analysis for Wiener Filter
  !   Claerbout, Fundamentals of Geophysical Data Processing for Levinson
  !   algorithm from Ettore / wikipedia (Levinson recursion)
  !=========================================
  subroutine wiener_filter(abcross,bbauto,filter,eps)
    ! declare variables
    integer                       :: ii,jj
    real                          :: eps,alpha
    real                          :: error,error_filter
    real,dimension(:)             :: abcross,bbauto
    real,dimension(:)             :: filter
    real,allocatable,dimension(:) :: forward

    ! allocate
    allocate(forward(n1)); forward = 0.

    ! add white noise to stabilize inversion
    bbauto(1) = (1+eps)*bbauto(1)

    ! start inversion / recursion
    ! start with first element and solve for next element iteratively
    filter(1)  = abcross(1) / bbauto(1)
    forward(1) = 1 / bbauto(1)
    do ii = 2,n1
      error = 0.
      error_filter = 0.
      ! calculate error
      do jj = 2,ii
        error = error + bbauto(ii-jj+2)*forward(jj-1)
      end do
      ! update forward vector
      ! note: this is similar to backward vector (due to symmetry assumption)
      ! alpha is a scaling term out front
      alpha = 1./(1-error**2)
      forward(1:ii) = alpha*forward(1:ii) - error*alpha*forward(ii:1:-1)
      ! find last error in filter
      do jj = 2,ii
        error_filter = error_filter + bbauto(ii-jj+2)*filter(jj-1)
      end do
      ! update filter
      filter(1:ii) = filter(1:ii) + (abcross(ii)-error_filter)*forward(ii:1:-1)
    end do

    ! clean up
    deallocate(forward)

  end subroutine

end module
